/* ------------------------------------------------------------------------------------
 * Affichage des variables d'environnement (i.e., rcriture de printenv)
 * Jean-Paul Rigault 
 * ------------------------------------------------------------------------------------
 */

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <malloc.h>

/* Le tableau de chanes de caractres contenant les variables d'environnement */
extern char **environ; 

/* Fontions definies plus loin */
static void print_all();
static void print_var(int argc, char *argv[]);

/* Programme principal
 * -------------------
 *
 * Invocation : printenv.exe [variable ...]
 */
int main(int argc, char *argv[])
{
    /* S'il n'y a aucun argument, on crit la totalit de environ, sinon on
     * crit les valeurs des variables demandes
     */
    if (argc == 1) print_all();
    else print_var(argc, argv);
            
    return 0;
}

/* Lister tout le contenu de environ
* ----------------------------------
*/
void print_all()
{
    char **env;

    for (env = environ; *env != NULL; ++env)
        printf("%s\n", *env);
}

/* Lister les valeurs des variables indiques en argument 
 * ------------------------------------------------------
 *
 * On suppose ici que la taille du tableau environ est plus grande que la
 * longueur de la liste d'arguments. Donc on parcourt une seule fois
 * l'environnement et on conserve dans un tableau de pointeurs (locs) l'adresse
 * des chanes qu'il faudra afficher.
 */
void print_var(int argc, char *argv[])
{
    char **locs = malloc(argc * sizeof(char *));
    int i;
    char **env;

    for (env = environ; *env != NULL; ++env)
    {
        char *pequal; 
        int len;				

        pequal = strchr(*env, '='); 	/* recherche du signe '=" */
        assert(pequal != 0);		/* on doit avoir trouv '=' ! */
        len = pequal - *env;		/* longueur du nom de variable */
        
        /* On regarde si la ligne courante correspond  l'une des variables que
         * l'on recherche.
         */
        for (i = 1; i < argc; ++i) 
        {
            if (strncmp(*env, argv[i], len) == 0) 
                locs[i] = pequal + 1;
        } 
    }

    /* Impression du rsultat */
    for (i = 1; i < argc; ++i) 
    {
        if (locs[i] != NULL) printf("%s\n", locs[i]);
    }
}
