/* ------------------------------------------------------------------------------------
 * Quelques fonctions utiles pour lsrec et copy
 * Jean-Paul Rigault 
 * ------------------------------------------------------------------------------------
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <string.h>

#include "util.h"


/* Le nom de fichier est-il celui d'un rpertoire ? */
int is_dir(const char *filename) 
{
    struct stat st;

    if (stat(filename, &st) == -1) return 0;
    return S_ISDIR(st.st_mode);
}

/* Le nom de fichier est-il "." ou ".." ? */
int is_dot_dir(const char *filename)
{
    return (filename[0] == '.')
        && (filename[1] == '\0' || (filename[1] == '.' && filename[2] == '\0'));
}

/* Nom de base d'un fichier */
const char *get_basename(const char *filename)
{
    const char *b = strrchr(filename, '/'); /* recherche du dernier car. '/' */
    return (b == NULL) ? filename : (b + 1);
}

/* Deux noms de fichiers dsignent-ils le mme fichier physique ? */
int same_file(const char *fn1, const char *fn2)
{
    struct stat fn1st;
    struct stat fn2st;

  if (stat(fn1, &fn1st) || stat(fn2, &fn2st)) return 0;

  /* Les deux fichiers existent. Ils sont identiques s'ils ont le mme i-numro
   * (st_ino) et rsident dan le mme systme de fichiers (st_dev) */
  return fn1st.st_dev == fn2st.st_dev && fn1st.st_ino == fn2st.st_ino;
}


/* Tester l'existence d'un fichier */
int file_exists(const char *filename)
{
    struct stat st;
  
    return !stat(filename, &st);  /* peut-tre un peu simpliste (ENOENT ?) */
}
